@@ -680,80 +680,4 @@ class BrowserUseAgent(Agent):
 
                 create_history_gif(task=self.task, history=self.history, output_path=output_path)
 
-            await self.close()
-
-    def _matches_action_type(self, action_type: str, allowed_pattern: str) -> bool:
-        """
-        Check if an action type matches an allowed pattern, supporting wildcards.
-        
-        Args:
-            action_type: The actual action type (e.g., "mcp.filesystem.read_file")
-            allowed_pattern: The pattern to match (e.g., "mcp.filesystem*")
-            
-        Returns:
-            True if the action type matches the pattern
-        """
-        if allowed_pattern.endswith('*'):
-            # Wildcard matching
-            prefix = allowed_pattern[:-1]
-            return action_type.startswith(prefix)
-        else:
-            # Exact matching
-            return action_type == allowed_pattern
-
-    def _is_action_parallel_allowed(self, action: ActionModel) -> bool:
-        """
-        Check if an action is allowed to be executed in parallel.
-        
-        Args:
-            action: The action to check
-            
-        Returns:
-            True if the action can be executed in parallel
-        """
-        action_data = action.model_dump(exclude_unset=True)
-        action_type = next(iter(action_data.keys())) if action_data else None
-
-        if not action_type:
-            return False
-
-        for allowed_pattern in self.allow_parallel_action_types:
-            if self._matches_action_type(action_type, allowed_pattern):
-                return True
-
-        return False
-
-    def _group_actions_for_parallel_execution(self, actions: list[ActionModel]) -> list[list[ActionModel]]:
-        """
-        Group consecutive actions that can be executed in parallel.
-        
-        Args:
-            actions: List of actions to group
-            
-        Returns:
-            List of action groups, where each group can be executed in parallel
-        """
-        if not actions:
-            return []
-
-        groups = []
-        current_group = [actions[0]]
-
-        for i in range(1, len(actions)):
-            current_action = actions[i]
-            previous_action = actions[i - 1]
-
-            # Check if both current and previous actions can be executed in parallel
-            if (self._is_action_parallel_allowed(current_action) and
-                    self._is_action_parallel_allowed(previous_action)):
-                # Add to current group
-                current_group.append(current_action)
-            else:
-                # Start a new group
-                groups.append(current_group)
-                current_group = [current_action]
-
-        # Add the last group
-        groups.append(current_group)
-
-        return groups
\ No newline at end of file
+            await self.close()
\ No newline at end of file
