@@ -29,6 +29,7 @@ from browser_use.tokens.service import TokenCost
 from vibe_surf.agents.browser_use_agent import BrowserUseAgent
 from vibe_surf.agents.report_writer_agent import ReportWriterAgent, ReportTaskResult
 from vibe_surf.agents.views import CustomAgentOutput
+from vibe_surf.utils import check_latest_vibesurf_version, get_vibesurf_version
 
 from vibe_surf.agents.prompts.vibe_surf_prompt import (
     VIBESURF_SYSTEM_PROMPT,
@@ -359,7 +360,9 @@ async def _vibesurf_agent_node_impl(state: VibeSurfState) -> VibeSurfState:
 
     # Create action model and agent output using VibeSurfTools
     vibesurf_agent = state.vibesurf_agent
-    ActionModel = vibesurf_agent.tools.registry.create_action_model()
+
+    vibesurf_action_names = vibesurf_agent.tools.get_all_action_names(exclude_actions=['mcp.', 'cpo.'])
+    ActionModel = vibesurf_agent.tools.registry.create_action_model(include_actions=vibesurf_action_names)
     if vibesurf_agent.settings.agent_mode == "thinking":
         AgentOutput = CustomAgentOutput.type_with_custom_actions(ActionModel)
     else:
@@ -393,13 +396,15 @@ async def _vibesurf_agent_node_impl(state: VibeSurfState) -> VibeSurfState:
             context_info.append(f"Generated Report: ❌ Failed - {state.generated_report_result.msg}\nPath: {state.generated_report_result.report_path}\n")
 
     context_str = "\n".join(context_info) if context_info else "No additional context available."
+    logger.debug("VibeSurf State Message:\n")
+    logger.debug(context_str)
     vibesurf_agent.message_history.append(UserMessage(content=context_str))
 
     try:
         # Get LLM response with action output format
         response = await vibesurf_agent.llm.ainvoke(vibesurf_agent.message_history, output_format=AgentOutput)
         parsed = response.completion
-        actions = parsed.action
+        action = parsed.action
         vibesurf_agent.message_history.append(
             AssistantMessage(content=json.dumps(response.completion.model_dump(exclude_none=True, exclude_unset=True),
                                                 ensure_ascii=False)))
@@ -407,15 +412,14 @@ async def _vibesurf_agent_node_impl(state: VibeSurfState) -> VibeSurfState:
         # Capture telemetry for parsed output
         import vibe_surf
         action_types = []
-        for action in actions:
-            action_data = action.model_dump(exclude_unset=True)
-            action_name = next(iter(action_data.keys())) if action_data else 'unknown'
-            action_types.append(action_name)
+        action_data = action.model_dump(exclude_unset=True)
+        action_name = next(iter(action_data.keys())) if action_data else 'unknown'
+        action_types.append(action_name)
         
         parsed_output_event = VibeSurfAgentParsedOutputEvent(
             version=vibe_surf.__version__,
             parsed_output=json.dumps(parsed.model_dump(exclude_none=True, exclude_unset=True), ensure_ascii=False),  # Limit size
-            action_count=len(actions),
+            action_count=1,
             action_types=action_types,
             model=getattr(vibesurf_agent.llm, 'model_name', None),
             session_id=state.session_id,
@@ -426,103 +430,95 @@ async def _vibesurf_agent_node_impl(state: VibeSurfState) -> VibeSurfState:
         # Log thinking if present
         if hasattr(parsed, 'thinking') and parsed.thinking:
             await log_agent_activity(state, agent_name, "thinking", parsed.thinking)
+        action_data = action.model_dump(exclude_unset=True)
+        action_name = next(iter(action_data.keys())) if action_data else 'unknown'
+        logger.info(f"🛠️ Processing VibeSurf action: {action_name}")
 
-        for i, action in enumerate(actions):
-            action_data = action.model_dump(exclude_unset=True)
-            action_name = next(iter(action_data.keys())) if action_data else 'unknown'
-            logger.info(f"🛠️ Processing VibeSurf action {i + 1}/{len(actions)}: {action_name}")
+        # Check for special routing actions
+        if action_name == 'execute_browser_use_agent':
+            # Route to browser task execution node
+            params = action_data[action_name]
+            state.browser_tasks = params.get('tasks', [])
+            state.current_action = 'execute_browser_use_agent'
+            state.action_params = params
+            state.current_step = "browser_task_execution"
 
-            # Check for special routing actions
-            if action_name == 'execute_browser_use_agent':
-                # Route to browser task execution node
-                params = action_data[action_name]
-                state.browser_tasks = params.get('tasks', [])
-                state.current_action = 'execute_browser_use_agent'
-                state.action_params = params
-                state.current_step = "browser_task_execution"
+            # Log agent activity
+            browser_tasks_md = []
+            for browser_task in state.browser_tasks:
+                bu_task = browser_task.get('task', "")
+                if bu_task:
+                    bu_task_tabid = browser_task.get('tab_id', "")
+                    browser_tasks_md.append(f"- [ ] Tab id: {bu_task_tabid} working {bu_task}")
+            browser_tasks_md = '\n'.join(browser_tasks_md)
+            agent_msg = f"Routing to browser task execution with  {len(state.browser_tasks)} browser tasks:\n\n{browser_tasks_md}"
+            await log_agent_activity(state, agent_name, "working", agent_msg)
+            logger.debug(agent_msg)
+            return state
 
-                # Log agent activity
-                browser_tasks_md = []
-                for browser_task in state.browser_tasks:
-                    bu_task = browser_task.get('task', "")
-                    if bu_task:
-                        browser_tasks_md.append(f"- [ ] {bu_task}")
-                browser_tasks_md = '\n'.join(browser_tasks_md)
-                agent_msg = f"Routing to browser task execution with  {len(state.browser_tasks)} browser tasks:\n\n{browser_tasks_md}"
-                await log_agent_activity(state, agent_name, "working", agent_msg)
-                logger.debug(agent_msg)
-                return state
+        elif action_name == 'execute_report_writer_agent':
+            # Route to report task execution node
+            params = action_data[action_name]
+            state.current_action = 'execute_report_writer_agent'
+            state.action_params = params
+            state.current_step = "report_task_execution"
+            report_task = params.get('task', "")
+            agent_msg = f"Routing to report generation with task:\n{report_task}"
+            await log_agent_activity(state, agent_name, "working", agent_msg)
+            return state
 
-            elif action_name == 'execute_report_writer_agent':
-                # Route to report task execution node
-                params = action_data[action_name]
-                state.current_action = 'execute_report_writer_agent'
-                state.action_params = params
-                state.current_step = "report_task_execution"
-                report_task = params.get('task', "")
-                agent_msg = f"Routing to report generation with task:\n{report_task}"
-                await log_agent_activity(state, agent_name, "working", agent_msg)
-                return state
+        elif action_name == 'task_done':
+            # Handle response/completion - direct to END
+            params = action_data[action_name]
+            response_content = params.get('response', 'Task completed!')
+            follow_tasks = params.get('suggestion_follow_tasks', [])
+            state.current_step = "END"
 
-            elif action_name == 'task_done':
-                # Handle response/completion - direct to END
-                params = action_data[action_name]
-                response_content = params.get('response', 'Task completed!')
-                follow_tasks = params.get('suggestion_follow_tasks', [])
-                state.current_step = "END"
+            # Format final response
+            final_response = f"{response_content}"
+            await log_agent_activity(state, agent_name, "result", final_response)
 
-                # Format final response
-                final_response = f"{response_content}"
-                await log_agent_activity(state, agent_name, "result", final_response)
+            if follow_tasks:
+                await log_agent_activity(state, agent_name, "suggestion_tasks",
+                                         '\n'.join(follow_tasks))
+                final_response += "\n\n## Suggested Follow-up Tasks:\n"
+                for j, task in enumerate(follow_tasks[:3], 1):
+                    final_response += f"{j}. {task}\n"
 
-                if follow_tasks:
-                    await log_agent_activity(state, agent_name, "suggestion_tasks",
-                                             '\n'.join(follow_tasks))
-                    final_response += "\n\n## Suggested Follow-up Tasks:\n"
-                    for j, task in enumerate(follow_tasks[:3], 1):
-                        final_response += f"{j}. {task}\n"
-
-                state.final_response = final_response
-                logger.debug(final_response)
-                state.is_complete = True
-                return state
+            state.final_response = final_response
+            logger.debug(final_response)
+            state.is_complete = True
+            return state
 
+        else:
+            if "todos" in action_name:
+                todo_content = await vibesurf_agent.file_system.read_file('todo.md')
+                action_msg = f"{action_name}:\n\n{todo_content}"
+                logger.debug(action_msg)
+                await log_agent_activity(state, agent_name, "working", action_msg)
             else:
-                if "todos" in action_name:
-                    todo_content = await vibesurf_agent.file_system.read_file('todo.md')
-                    action_msg = f"{action_name}:\n\n{todo_content}"
-                    logger.debug(action_msg)
-                    await log_agent_activity(state, agent_name, "working", action_msg)
-                else:
-                    action_msg = f"**⚡ Actions:**\n"
-                    action_msg += f"```json\n{json.dumps(action_data, indent=2, ensure_ascii=False)}\n```"
-                    logger.debug(action_msg)
-                    await log_agent_activity(state, agent_name, "working", action_msg)
+                action_msg = f"**⚡ Actions:**\n"
+                action_msg += f"```json\n{json.dumps(action_data, indent=2, ensure_ascii=False)}\n```"
+                logger.debug(action_msg)
+                await log_agent_activity(state, agent_name, "working", action_msg)
 
-                result = await vibesurf_agent.tools.act(
-                    action=action,
-                    browser_manager=vibesurf_agent.browser_manager,
-                    llm=vibesurf_agent.llm,
-                    file_system=vibesurf_agent.file_system,
-                )
-                if action_name in ['skill_search', 'skill_crawl', 'skill_summary', 'skill_deep_research']:
-                    state.current_step = "END"
-                    # Format final response
-                    final_response = f"{result.extracted_content}" or f"{result.error}"
-                    state.final_response = final_response
-                    logger.debug(final_response)
-                    state.is_complete = True
-                else:
-                    state.current_step = "vibesurf_agent"
+            result = await vibesurf_agent.tools.act(
+                action=action,
+                browser_manager=vibesurf_agent.browser_manager,
+                llm=vibesurf_agent.llm,
+                file_system=vibesurf_agent.file_system,
+            )
 
-                if result.extracted_content:
-                    vibesurf_agent.message_history.append(
-                        UserMessage(content=f'Action result:\n{result.extracted_content}'))
-                    await log_agent_activity(state, agent_name, "result", result.extracted_content)
+            state.current_step = "vibesurf_agent"
 
-                if result.error:
-                    vibesurf_agent.message_history.append(UserMessage(content=f'Action error:\n{result.error}'))
-                    await log_agent_activity(state, agent_name, "error", result.error)
+            if result.extracted_content:
+                vibesurf_agent.message_history.append(
+                    UserMessage(content=f'Action result:\n{result.extracted_content}'))
+                await log_agent_activity(state, agent_name, "result", result.extracted_content)
+
+            if result.error:
+                vibesurf_agent.message_history.append(UserMessage(content=f'Action error:\n{result.error}'))
+                await log_agent_activity(state, agent_name, "error", result.error)
 
         return state
 
@@ -629,7 +625,7 @@ async def execute_parallel_browser_tasks(state: VibeSurfState) -> List[BrowserTa
         task_description = task_info.get('task', '')
         if not task_description:
             continue
-        target_id = task_info.get('target_id', None)
+        target_id = task_info.get('tab_id', None)
         register_sessions.append(
             state.vibesurf_agent.browser_manager.register_agent(agent_id, target_id=target_id)
         )
@@ -638,11 +634,11 @@ async def execute_parallel_browser_tasks(state: VibeSurfState) -> List[BrowserTa
 
     vibesurf_tools = state.vibesurf_agent.tools
     bu_tools = BrowserUseTools()
-    for mcp_server_name, mcp_client in vibesurf_tools.mcp_clients.items():
-        await mcp_client.register_to_tools(
-            tools=bu_tools,
-            prefix=f"mcp.{mcp_server_name}."
-        )
+    # for mcp_server_name, mcp_client in vibesurf_tools.mcp_clients.items():
+    #     await mcp_client.register_to_tools(
+    #         tools=bu_tools,
+    #         prefix=f"mcp.{mcp_server_name}."
+    #     )
     bu_tasks = [None] * len(pending_tasks)
     for i, task_info in enumerate(pending_tasks):
         agent_id = f"bu_agent-{task_id}-{i + 1:03d}"
@@ -761,7 +757,7 @@ async def execute_parallel_browser_tasks(state: VibeSurfState) -> List[BrowserTa
     finally:
         # Remove agents from control tracking and cleanup browser sessions
         for i, agent_id in enumerate(bu_agent_ids):
-            if not isinstance(pending_tasks[i], list):
+            if pending_tasks[i].get('tab_id', None) is None:
                 await state.vibesurf_agent.browser_manager.unregister_agent(agent_id, close_tabs=True)
             if state.vibesurf_agent and hasattr(state.vibesurf_agent, '_running_agents'):
                 state.vibesurf_agent._running_agents.pop(agent_id, None)
@@ -796,11 +792,11 @@ async def execute_single_browser_tasks(state: VibeSurfState) -> BrowserTaskResul
     try:
         vibesurf_tools = state.vibesurf_agent.tools
         bu_tools = BrowserUseTools()
-        for mcp_server_name, mcp_client in vibesurf_tools.mcp_clients.items():
-            await mcp_client.register_to_tools(
-                tools=bu_tools,
-                prefix=f"mcp.{mcp_server_name}."
-            )
+        # for mcp_server_name, mcp_client in vibesurf_tools.mcp_clients.items():
+        #     await mcp_client.register_to_tools(
+        #         tools=bu_tools,
+        #         prefix=f"mcp.{mcp_server_name}."
+        #     )
         available_file_paths = []
         if task_files:
             for task_file in task_files:
@@ -827,7 +823,7 @@ async def execute_single_browser_tasks(state: VibeSurfState) -> BrowserTaskResul
             await main_browser_session.get_or_create_cdp_session(target_id=target_id)
         else:
             new_target = await main_browser_session.cdp_client.send.Target.createTarget(
-                params={'url': 'about:blank'})
+                params={'url': 'chrome://newtab/'})
             target_id = new_target["targetId"]
             await main_browser_session.get_or_create_cdp_session(target_id=target_id)
         agent = BrowserUseAgent(
@@ -1055,7 +1051,8 @@ class VibeSurfAgent:
             browser_manager: BrowserManager,
             tools: VibeSurfTools,
             workspace_dir: str = "./workspace",
-            settings: Optional[VibeSurfAgentSettings] = None
+            settings: Optional[VibeSurfAgentSettings] = None,
+            extend_system_prompt: Optional[str] = None,
     ):
         """Initialize VibeSurfAgent with required components"""
         self.llm: BaseChatModel = llm
@@ -1066,7 +1063,7 @@ class VibeSurfAgent:
         self.tools: VibeSurfTools = tools
         self.workspace_dir = workspace_dir
         os.makedirs(self.workspace_dir, exist_ok=True)
-
+        self.extend_system_prompt = extend_system_prompt
         self.cur_session_id = None
         self.file_system: Optional[CustomFileSystem] = None
         self.message_history = []
@@ -1646,25 +1643,9 @@ Please continue with your assigned work, incorporating this guidance only if it'
 
             if not self.message_history:
                 vibesurf_system_prompt = VIBESURF_SYSTEM_PROMPT
-                if self.settings.agent_mode == "thinking":
-                    vibesurf_system_prompt += """
-You must ALWAYS respond with a valid JSON in this exact format:
-{{
-  "thinking": "A structured <think>-style reasoning.",
-  "action":[{{"task_done": {{ }}, // ... more actions in sequence]
-}}
+                if self.extend_system_prompt:
+                    vibesurf_system_prompt += f"\n Extend System Prompt provided by user:\n {self.extend_system_prompt}"
 
-Action list should NEVER be empty.
-                    """
-                else:
-                    vibesurf_system_prompt += """
-You must ALWAYS respond with a valid JSON in this exact format:
-{{
-  "action":[{{"task_done": {{ }}, // ... more actions in sequence]
-}}
-
-Action list should NEVER be empty.
-                    """
                 self.message_history.append(SystemMessage(content=vibesurf_system_prompt))
 
             # Format processed upload files for prompt
@@ -1685,6 +1666,18 @@ Action list should NEVER be empty.
             }
             self.activity_logs.append(activity_entry)
 
+            latest_version = await check_latest_vibesurf_version()
+            current_version = get_vibesurf_version()
+            if latest_version and latest_version != current_version:
+                update_msg = f'📦 Newer version of vibesurf available: {latest_version} (current: {current_version}). \nUpgrade with: \n`uv pip install vibesurf -U`\nor\nDownload [Windows Installer](https://github.com/vibesurf-ai/VibeSurf/releases/latest/download/vibesurf-windows-x64.exe).\n\nYou can find more information at [release page](https://github.com/vibesurf-ai/VibeSurf/releases).'
+                logger.debug(update_msg)
+                activity_update_tip = {
+                    "agent_name": 'System',
+                    "agent_status": 'tip',  # working, result, error
+                    "agent_msg": update_msg
+                }
+                self.activity_logs.append(activity_update_tip)
+
             # Initialize state first (needed for file processing)
             initial_state = VibeSurfState(
                 original_task=task,
@@ -1728,7 +1721,7 @@ Action list should NEVER be empty.
             completion_event = VibeSurfAgentTelemetryEvent(
                 version=vibe_surf.__version__,
                 action='task_completed',
-                task_description=task[:200] if task else None,
+                task_description=task if task else None,
                 model=getattr(self.llm, 'model_name', None),
                 model_provider=getattr(self.llm, 'provider', None),
                 duration_seconds=duration,
