@@ -5,7 +5,7 @@ import os
 import pdb
 from pathlib import Path
 from typing import TYPE_CHECKING, Any, Literal, Self, Union, cast, Optional
-
+from cdp_use.cdp.target import AttachedToTargetEvent, SessionID, TargetID
 from browser_use.browser.session import BrowserSession, CDPSession
 from pydantic import Field
 from browser_use.browser.events import (
@@ -206,7 +206,7 @@ class AgentBrowserSession(BrowserSession):
             assert self._cdp_client_root is not None
             await self._cdp_client_root.start()
             await self._cdp_client_root.send.Target.setAutoAttach(
-                params={'autoAttach': True, 'waitForDebuggerOnStart': False, 'flatten': True}
+                params={'autoAttach': False, 'waitForDebuggerOnStart': False, 'flatten': True}
             )
             self.logger.debug('CDP client connected successfully')
 
@@ -228,39 +228,9 @@ class AgentBrowserSession(BrowserSession):
             from browser_use.utils import is_new_tab_page
 
             # Collect all targets that need redirection
-            redirected_targets = []
-            redirect_sessions = {}  # Store sessions created for redirection to potentially reuse
-            for target in page_targets:
-                target_url = target.get('url', '')
-                if is_new_tab_page(target_url) and target_url != '':
-                    # Redirect chrome://newtab to about:blank to avoid JS issues preventing driving chrome://newtab
-                    target_id = target['targetId']
-                    self.logger.debug(f'🔄 Redirecting {target_url} to about:blank for target {target_id}')
-                    try:
-                        # Create a CDP session for redirection (minimal domains to avoid duplicate event handlers)
-                        # Only enable Page domain for navigation, avoid duplicate event handlers
-                        redirect_session = await CDPSession.for_target(self._cdp_client_root, target_id,
-                                                                       domains=['Page'])
-                        # Navigate to about:blank
-                        await redirect_session.cdp_client.send.Page.navigate(
-                            params={'url': ''}, session_id=redirect_session.session_id
-                        )
-                        redirected_targets.append(target_id)
-                        redirect_sessions[target_id] = redirect_session  # Store for potential reuse
-                        # Update the target's URL in our list for later use
-                        target['url'] = ''
-                        # Small delay to ensure navigation completes
-                        await asyncio.sleep(0.1)
-                    except Exception as e:
-                        self.logger.warning(f'Failed to redirect {target_url} to about:blank: {e}')
-
-            # Log summary of redirections
-            if redirected_targets:
-                self.logger.debug(f'Redirected {len(redirected_targets)} chrome://newtab pages to about:blank')
-
             if not page_targets:
                 # No pages found, create a new one
-                new_target = await self._cdp_client_root.send.Target.createTarget(params={'url': ''})
+                new_target = await self._cdp_client_root.send.Target.createTarget(params={'url': 'chrome://newtab/'})
                 target_id = new_target['targetId']
                 self.logger.debug(f'📄 Created new blank page with target ID: {target_id}')
             else:
@@ -268,14 +238,8 @@ class AgentBrowserSession(BrowserSession):
                 target_id = [page for page in page_targets if page.get('type') == 'page'][0]['targetId']
                 self.logger.debug(f'📄 Using existing page with target ID: {target_id}')
 
-            # Store the current page target ID and add to pool
-            # Reuse redirect session if available, otherwise create new one
-            if target_id in redirect_sessions:
-                self.logger.debug(f'Reusing redirect session for target {target_id}')
-                self.agent_focus = redirect_sessions[target_id]
-            else:
-                # For the initial connection, we'll use the shared root WebSocket
-                self.agent_focus = await CDPSession.for_target(self._cdp_client_root, target_id, new_socket=False)
+            self.agent_focus = await CDPSession.for_target(self._cdp_client_root, target_id)
+
             if self.agent_focus:
                 self._cdp_session_pool[target_id] = self.agent_focus
 
@@ -297,13 +261,12 @@ class AgentBrowserSession(BrowserSession):
             for idx, target in enumerate(page_targets):
                 target_url = target.get('url', '')
                 self.logger.debug(f'Dispatching TabCreatedEvent for initial tab {idx}: {target_url}')
-                await self.event_bus.dispatch(TabCreatedEvent(url=target_url, target_id=target['targetId']))
+                self.event_bus.dispatch(TabCreatedEvent(url=target_url, target_id=target['targetId']))
 
             # Dispatch initial focus event
             if page_targets:
                 initial_url = page_targets[0].get('url', '')
-                await self.event_bus.dispatch(
-                    AgentFocusChangedEvent(target_id=page_targets[0]['targetId'], url=initial_url))
+                self.event_bus.dispatch(AgentFocusChangedEvent(target_id=page_targets[0]['targetId'], url=initial_url))
                 self.logger.debug(f'Initial agent focus set to tab 0: {initial_url}')
 
         except Exception as e:
@@ -327,10 +290,9 @@ class AgentBrowserSession(BrowserSession):
         assigned_target_ids = self._cdp_session_pool.keys()
         if target_id not in assigned_target_ids:
             self.logger.info(f"Agent {self.id}: Assigned target {target_id}")
-            self.agent_focus = await CDPSession.for_target(self._cdp_client_root, target_id, new_socket=True,
-                                                           cdp_url=self.cdp_url)
-            await self.agent_focus.cdp_client.send.Target.activateTarget(
-                params={'targetId': target_id})
+            self.agent_focus = await CDPSession.for_target(self._cdp_client_root, target_id)
+            # await self.agent_focus.cdp_client.send.Target.activateTarget(
+            #     params={'targetId': target_id})
             await self.agent_focus.cdp_client.send.Runtime.runIfWaitingForDebugger(
                 session_id=self.agent_focus.session_id)
             self._cdp_session_pool[target_id] = self.agent_focus
@@ -515,7 +477,7 @@ class AgentBrowserSession(BrowserSession):
             if new_tab:
                 # Create new tab directly via CDP - no event system overhead
                 result = await self._cdp_client_root.send.Target.createTarget(
-                    params={'url': 'about:blank', 'newWindow': False, 'background': False}
+                    params={'url': 'chrome://newtab/', 'newWindow': False, 'background': False}
                 )
                 target_id = result['targetId']
 
@@ -566,34 +528,19 @@ class AgentBrowserSession(BrowserSession):
         finally:
             return target_id
 
-    async def _wait_for_stable_network(self):
+    async def _wait_for_stable_network(self, target_id=None, max_attempt=3):
         """Wait for page stability - simplified for CDP-only branch."""
-        start_time = time.time()
-        page_url = await self.get_current_page_url()
-        not_a_meaningful_website = page_url.lower().split(':', 1)[0] not in ('http', 'https')
-
-        # Wait for page stability using browser profile settings (main branch pattern)
-        if not not_a_meaningful_website:
-            self.logger.debug('🔍 DOMWatchdog.on_BrowserStateRequestEvent: ⏳ Waiting for page stability...')
+        cdp_session = await self.get_or_create_cdp_session(target_id=target_id)
+        for _ in range(max_attempt):
             try:
-                # Apply minimum wait time first (let page settle)
-                min_wait = self.browser_profile.minimum_wait_page_load_time
-                if min_wait > 0:
-                    self.logger.debug(f'⏳ Minimum wait: {min_wait}s')
-                    await asyncio.sleep(min_wait)
-
-                # Apply network idle wait time (for dynamic content like iframes)
-                network_idle_wait = self.browser_profile.wait_for_network_idle_page_load_time
-                if network_idle_wait > 0:
-                    self.logger.debug(f'⏳ Network idle wait: {network_idle_wait}s')
-                    await asyncio.sleep(network_idle_wait)
-
-                elapsed = time.time() - start_time
-                self.logger.debug(f'✅ Page stability wait completed in {elapsed:.2f}s')
-            except Exception as e:
-                self.logger.warning(
-                    f'🔍 DOMWatchdog.on_BrowserStateRequestEvent: Network waiting failed: {e}, continuing anyway...'
+                ready_state = await cdp_session.cdp_client.send.Runtime.evaluate(
+                    params={'expression': 'document.readyState'}, session_id=cdp_session.session_id
                 )
+                if ready_state and ready_state.get("value", "loading") == "complete":
+                    break
+            except Exception:
+                pass
+            await asyncio.sleep(1.0)
 
     async def take_screenshot(self, target_id: Optional[str] = None,
                               path: str | None = None,
@@ -609,14 +556,7 @@ class AgentBrowserSession(BrowserSession):
         """
 
         cdp_session = await self.get_or_create_cdp_session(target_id, focus=False)
-        await self._wait_for_stable_network()
-
-        try:
-            ready_state = await cdp_session.cdp_client.send.Runtime.evaluate(
-                params={'expression': 'document.readyState'}, session_id=cdp_session.session_id
-            )
-        except Exception:
-            pass
+        await self._wait_for_stable_network(target_id)
 
         try:
             import base64
@@ -672,14 +612,7 @@ class AgentBrowserSession(BrowserSession):
         """
 
         cdp_session = await self.get_or_create_cdp_session(target_id, focus=False)
-        await self._wait_for_stable_network()
-
-        try:
-            ready_state = await cdp_session.cdp_client.send.Runtime.evaluate(
-                params={'expression': 'document.readyState'}, session_id=cdp_session.session_id
-            )
-        except Exception:
-            pass
+        await self._wait_for_stable_network(target_id)
 
         try:
             import base64
@@ -717,6 +650,69 @@ class AgentBrowserSession(BrowserSession):
             self.logger.error(f'Concurrent screenshot failed: {type(e).__name__}: {e}')
             raise
 
+    async def get_or_create_cdp_session(
+            self, target_id: TargetID | None = None, focus: bool = True
+    ) -> CDPSession:
+        """Get or create a CDP session for a target.
+
+        Args:
+                target_id: Target ID to get session for. If None, uses current agent focus.
+                focus: If True, switches agent focus to this target. If False, just returns session without changing focus.
+
+        Returns:
+                CDPSession for the specified target.
+        """
+        assert self.cdp_url is not None, 'CDP URL not set - browser may not be configured or launched yet'
+        assert self._cdp_client_root is not None, 'Root CDP client not initialized - browser may not be connected yet'
+        assert self.agent_focus is not None, 'CDP session not initialized - browser may not be connected yet'
+
+        # If no target_id specified, use the current target_id
+        if target_id is None:
+            target_id = self.agent_focus.target_id
+
+        # Check if we already have a session for this target in the pool
+        if target_id in self._cdp_session_pool:
+            session = self._cdp_session_pool[target_id]
+            if focus and self.agent_focus.target_id != target_id:
+                self.logger.debug(
+                    f'[get_or_create_cdp_session] Switching agent focus from {self.agent_focus.target_id} to {target_id}'
+                )
+                self.agent_focus = session
+            if focus:
+                # await session.cdp_client.send.Target.activateTarget(params={'targetId': session.target_id})
+                await session.cdp_client.send.Runtime.runIfWaitingForDebugger(session_id=session.session_id)
+            # else:
+            # self.logger.debug(f'[get_or_create_cdp_session] Reusing existing session for {target_id} (focus={focus})')
+            return session
+
+        # If it's the current focus target, return that session
+        if self.agent_focus.target_id == target_id:
+            self._cdp_session_pool[target_id] = self.agent_focus
+            return self.agent_focus
+
+        session = await CDPSession.for_target(
+            self._cdp_client_root,
+            target_id
+        )
+        self._cdp_session_pool[target_id] = session
+        # log length of _cdp_session_pool
+        self.logger.debug(f'[get_or_create_cdp_session] new _cdp_session_pool length: {len(self._cdp_session_pool)}')
+
+        # Only change agent focus if requested
+        if focus:
+            self.logger.debug(
+                f'[get_or_create_cdp_session] Switching agent focus from {self.agent_focus.target_id} to {target_id}'
+            )
+            self.agent_focus = session
+            # await session.cdp_client.send.Target.activateTarget(params={'targetId': session.target_id})
+            await session.cdp_client.send.Runtime.runIfWaitingForDebugger(session_id=session.session_id)
+        else:
+            self.logger.debug(
+                f'[get_or_create_cdp_session] Created session for {target_id} without changing focus (still on {self.agent_focus.target_id})'
+            )
+
+        return session
+
     async def get_html_content(self, target_id: Optional[str] = None) -> str:
         """
         Get html content of current page
@@ -724,14 +720,7 @@ class AgentBrowserSession(BrowserSession):
         """
 
         cdp_session = await self.get_or_create_cdp_session(target_id, focus=False)
-        await self._wait_for_stable_network()
-
-        try:
-            ready_state = await cdp_session.cdp_client.send.Runtime.evaluate(
-                params={'expression': 'document.readyState'}, session_id=cdp_session.session_id
-            )
-        except Exception:
-            await self._wait_for_stable_network()
+        await self._wait_for_stable_network(target_id)
 
         try:
             # Get the HTML content
