@@ -22,7 +22,9 @@ import numpy as np
 from typing import Optional, Tuple, List, Any
 import io
 
-logger = logging.getLogger(__name__)
+from vibe_surf.logger import get_logger
+
+logger = get_logger(__name__)
 
 
 # List of common font file names (Prioritize preferred ones first)
@@ -601,7 +603,7 @@ def highlight_screenshot(screenshot_base64: str, elements: List[List[Any]]) -> s
 
         # --- Default if all corners overlap (Logic unchanged) ---
         if not found_non_overlapping_spot:
-            logger.debug(f"Could not avoid label overlap for index {highlight_index}. Defaulting to top-left.")
+            # logger.debug(f"Could not avoid label overlap for index {highlight_index}. Defaulting to top-left.")
             chosen_label_bg_box, potential_text_ref_pos = calculate_label_placement(
                 corner='top_left',  # Default corner from original code
                 outline_box=draw_box_outline,
@@ -618,7 +620,7 @@ def highlight_screenshot(screenshot_base64: str, elements: List[List[Any]]) -> s
                     potential_text_ref_pos[0] + w_padding // 2,
                     potential_text_ref_pos[1] + h_padding // 2)  # ** OFFSET FROM ORIGINAL CODE **
             else:
-                logger.debug(f"Default top-left placement failed for index {highlight_index}. Skipping label.")
+                # logger.debug(f"Default top-left placement failed for index {highlight_index}. Skipping label.")
                 chosen_label_bg_box = None
                 chosen_text_pos = None
 
@@ -689,21 +691,14 @@ def create_highlighted_screenshot(
         for element_id, element in selector_map.items():
             try:
                 # Use snapshot bounds (document coordinates) if available, otherwise absolute_position
-                bounds = None
-                if element.snapshot_node and element.snapshot_node.bounds:
-                    bounds = element.snapshot_node.bounds
-                elif element.absolute_position:
-                    bounds = element.absolute_position
+                bounds = element.absolute_position
 
-                if not bounds:
-                    continue
-
-                # Convert from CSS pixels to device pixels for screenshot coordinates
-                # Note: bounds are already in CSS pixels, screenshot is in device pixels
-                x1 = int((bounds.x - viewport_offset_x) * device_pixel_ratio)
-                y1 = int((bounds.y - viewport_offset_y) * device_pixel_ratio)
-                x2 = int((bounds.x + bounds.width - viewport_offset_x) * device_pixel_ratio)
-                y2 = int((bounds.y + bounds.height - viewport_offset_y) * device_pixel_ratio)
+                # Scale coordinates from CSS pixels to device pixels for screenshot
+                # The screenshot is captured at device pixel resolution, but coordinates are in CSS pixels
+                x1 = int(bounds.x * device_pixel_ratio)
+                y1 = int(bounds.y * device_pixel_ratio)
+                x2 = int((bounds.x + bounds.width) * device_pixel_ratio)
+                y2 = int((bounds.y + bounds.height) * device_pixel_ratio)
 
                 # Ensure coordinates are within image bounds
                 img_width, img_height = image.size
