@@ -154,11 +154,11 @@ class VibeSurfApp {
 
   async initializeUIManager() {
     this.uiManager = new VibeSurfUIManager(this.sessionManager, this.apiClient);
-    
+
     // Initialize UI with loaded data
     await this.uiManager.initialize();
     
-    console.log('[VibeSurf] UI manager initialized');
+    console.log('[VibeSurf] UI manager initialized successfully');
   }
 
   setupErrorHandling() {
@@ -189,6 +189,12 @@ class VibeSurfApp {
     // Periodic backend health check
     setInterval(async () => {
       try {
+        // Check if apiClient exists and is initialized
+        if (!this.apiClient || typeof this.apiClient.healthCheck !== 'function') {
+          console.warn('[VibeSurf] Health check skipped - API client not available');
+          return;
+        }
+        
         const healthCheck = await this.apiClient.healthCheck();
         
         if (healthCheck.status === 'healthy') {
@@ -334,7 +340,7 @@ class VibeSurfApp {
             <div class="input-container">
               <div class="input-main">
                 <div class="textarea-container">
-                  <textarea id="task-input" class="task-input" placeholder="Describe your browsing task..." rows="3"></textarea>
+                  <textarea id="task-input" class="task-input" placeholder="Ask anything (/ for skills, @ to specify tab)" rows="3"></textarea>
                   <div class="input-actions">
                     <button id="attach-file-btn" class="action-btn attach-btn" title="Attach Files">
                       <svg width="16" height="16" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
@@ -413,18 +419,37 @@ class VibeSurfApp {
 
   // Cleanup method
   destroy() {
+    // Prevent multiple cleanup calls
+    if (this.isDestroying || !this.isInitialized) {
+      console.log('[VibeSurf] Cleanup already in progress or app not initialized, skipping...');
+      return;
+    }
+    
+    this.isDestroying = true;
     console.log('[VibeSurf] Cleaning up application...');
     
-    if (this.uiManager) {
-      this.uiManager.destroy();
+    try {
+      if (this.uiManager) {
+        this.uiManager.destroy();
+        this.uiManager = null;
+      }
+      
+      if (this.sessionManager) {
+        this.sessionManager.destroy();
+        this.sessionManager = null;
+      }
+      
+      if (this.apiClient) {
+        this.apiClient = null;
+      }
+      
+      this.isInitialized = false;
+      console.log('[VibeSurf] Application cleanup complete');
+    } catch (error) {
+      console.error('[VibeSurf] Error during cleanup:', error);
+    } finally {
+      this.isDestroying = false;
     }
-    
-    if (this.sessionManager) {
-      this.sessionManager.destroy();
-    }
-    
-    this.isInitialized = false;
-    console.log('[VibeSurf] Application cleanup complete');
   }
 
   // Get application status
@@ -456,11 +481,21 @@ document.addEventListener('DOMContentLoaded', async () => {
 
 // Handle page unload
 window.addEventListener('beforeunload', () => {
-  if (window.vibeSurfApp) {
+  if (window.vibeSurfApp && window.vibeSurfApp.isInitialized && !window.vibeSurfApp.isDestroying) {
+    console.log('[VibeSurf] Page unloading, cleaning up...');
     window.vibeSurfApp.destroy();
   }
 });
 
+// Handle visibility change to prevent unnecessary cleanup
+document.addEventListener('visibilitychange', () => {
+  if (document.visibilityState === 'hidden') {
+    console.log('[VibeSurf] Page hidden, but not cleaning up (might be tab switch)');
+  } else if (document.visibilityState === 'visible') {
+    console.log('[VibeSurf] Page visible again');
+  }
+});
+
 // Make app accessible for debugging
 if (typeof window !== 'undefined') {
   window.VibeSurfApp = VibeSurfApp;
@@ -482,6 +517,12 @@ chrome.runtime.onMessage.addListener((message, sender, sendResponse) => {
           .catch(error => sendResponse({ success: false, error: error.message }));
         return true; // Keep message channel open for async response
         
+      case 'MICROPHONE_PERMISSION_RESULT':
+        console.log('[VibeSurf] Received microphone permission result:', message);
+        // This message is typically handled by voice recorder, just acknowledge
+        sendResponse({ acknowledged: true });
+        break;
+        
       default:
         console.warn('[VibeSurf] Unknown message type:', message.type);
     }
