@@ -7,7 +7,7 @@ class VibeSurfSessionManager {
     this.currentSession = null;
     this.activityLogs = [];
     this.pollingInterval = null;
-    this.pollingFrequency = 1000; // 1 second
+    this.pollingFrequency = 300; // 300ms for faster response
     this.isPolling = false;
     this.eventListeners = new Map();
     
@@ -200,14 +200,37 @@ class VibeSurfSessionManager {
       throw new Error('No active session. Please create a session first.');
     }
 
-    const taskPayload = {
-      session_id: this.currentSession.id,
-      ...taskData
-    };
-
     try {
+      // Stop any existing polling before starting new task
+      this.stopActivityPolling();
+      
+      // Reset activity logs for new task to ensure proper index synchronization
+      this.activityLogs = [];
+      
+      // Sync with server logs to get the correct starting state
+      try {
+        await this.syncActivityLogsFromServer();
+      } catch (error) {
+        this.activityLogs = [];
+      }
+
+      const taskPayload = {
+        session_id: this.currentSession.id,
+        ...taskData
+      };
+
       const response = await this.apiClient.submitTask(taskPayload);
       
+      // Check if the response indicates LLM connection failure
+      if (response && response.success === false && response.error === 'llm_connection_failed') {
+        console.log('[SessionManager] LLM connection failed, emitting taskError');
+        this.emit('taskError', {
+          error: response,
+          sessionId: this.currentSession.id
+        });
+        throw new Error(response.message || 'LLM connection failed');
+      }
+      
       // Update current session with task info
       this.currentSession.currentTask = {
         taskId: response.task_id,
@@ -217,12 +240,12 @@ class VibeSurfSessionManager {
         submittedAt: new Date().toISOString()
       };
 
-      // Start activity polling
-      this.startActivityPolling();
-
       // Store updated session
       await this.storeSessionData();
 
+      // Start polling after task submission and sync
+      this.startActivityPolling();
+
       this.emit('taskSubmitted', {
         sessionId: this.currentSession.id,
         task: this.currentSession.currentTask,
@@ -270,6 +293,13 @@ class VibeSurfSessionManager {
         await this.storeSessionData();
       }
 
+      // Sync activity logs before resuming polling to ensure index consistency
+      try {
+        await this.syncActivityLogsFromServer();
+      } catch (error) {
+        // Continue with existing logs if sync fails
+      }
+
       // Restart polling when task is resumed
       this.startActivityPolling();
 
@@ -295,6 +325,13 @@ class VibeSurfSessionManager {
 
       // Stop polling when task is stopped
       this.stopActivityPolling();
+      
+      // Sync final activity logs to capture any termination messages
+      try {
+        await this.syncActivityLogsFromServer();
+      } catch (error) {
+        // Continue if sync fails
+      }
 
       this.emit('taskStopped', { sessionId: this.currentSession?.id, response });
       
@@ -306,6 +343,24 @@ class VibeSurfSessionManager {
     }
   }
 
+  async addNewTaskToPaused(newTaskDescription) {
+    try {
+      const response = await this.apiClient.addNewTask(newTaskDescription);
+      
+      this.emit('newTaskAdded', {
+        sessionId: this.currentSession?.id,
+        newTask: newTaskDescription,
+        response
+      });
+      
+      return response;
+    } catch (error) {
+      console.error('[SessionManager] Add new task failed:', error);
+      this.emit('taskError', { error: error.message, action: 'add_new_task' });
+      throw error;
+    }
+  }
+
   // Activity polling
   startActivityPolling() {
     if (this.isPolling) {
@@ -339,7 +394,6 @@ class VibeSurfSessionManager {
     }
 
     try {
-      // ✅ 使用当前logs数量作为message_index，确保获取下一个预期的log
       const requestIndex = this.activityLogs.length;
       
       console.log(`[SessionManager] 🔄 Polling activity at index ${requestIndex}, current logs: ${this.activityLogs.length}`);
@@ -353,26 +407,24 @@ class VibeSurfSessionManager {
       // Check both possible response formats
       const activityLog = response?.activity_log || response?.data?.activity_log;
       const totalAvailable = response?.total_available || response?.data?.total_available;
-      
-      // ✅ 关键逻辑：只有当获取到新log且与上一个不同时才处理
+
       if (response && activityLog) {
         const prevActivityLog = this.activityLogs.length > 0 ? this.activityLogs[this.activityLogs.length - 1] : null;
-        
-        // 检查是否为新的、不重复的activity log
+
         const isNewLog = !prevActivityLog || !this.areLogsEqual(prevActivityLog, activityLog);
         
         if (isNewLog) {
           // New activity log received
           const newLog = { ...activityLog };
           
-          // Add timestamp if not present
+          // Add timestamp if not present - this should now be handled by UI
           if (!newLog.timestamp) {
             newLog.timestamp = new Date().toISOString();
           }
           
           this.activityLogs.push(newLog);
-          
-          console.log(`[SessionManager] ✅ New unique activity received: ${newLog.agent_name} - ${newLog.agent_status}`);
+
+          console.log(`[SessionManager] ✅ New activity received: ${newLog.agent_name} - ${newLog.agent_status}`);
 
           await this.handleActivityUpdate(newLog);
 
@@ -383,8 +435,7 @@ class VibeSurfSessionManager {
           });
 
           // Check if task is completed or terminated
-          const terminalStatuses = ['done', 'completed', 'finished', 'error', 'failed',
-                                  'terminated', 'stopped', 'cancelled', 'aborted'];
+          const terminalStatuses = ['done'];
           
           if (terminalStatuses.includes(newLog.agent_status?.toLowerCase())) {
             this.stopActivityPolling();
@@ -430,7 +481,6 @@ class VibeSurfSessionManager {
     }
   }
 
-  // ✅ 新增：比较两个activity log是否相等的辅助方法
   areLogsEqual(log1, log2) {
     if (!log1 || !log2) return false;
     
@@ -457,7 +507,7 @@ class VibeSurfSessionManager {
         if (missingLogs.length > 0) {
           
           for (const log of missingLogs) {
-            // Add timestamp if not present
+            // Add timestamp if not present - this should now be handled by UI
             if (!log.timestamp) {
               log.timestamp = new Date().toISOString();
             }
@@ -477,6 +527,47 @@ class VibeSurfSessionManager {
     }
   }
 
+  async syncActivityLogsFromServer() {
+    if (!this.currentSession) return;
+    
+    try {
+      console.log(`[SessionManager] 🔄 Syncing all activity logs from server for session: ${this.currentSession.id}`);
+      
+      // Get all activity logs from server
+      const response = await this.apiClient.getSessionActivity(this.currentSession.id);
+      
+      // Check both possible response formats
+      const serverLogs = response?.activity_logs || response?.data?.activity_logs || [];
+      
+      if (Array.isArray(serverLogs)) {
+        // 完全同步：用服务器端的logs替换本地logs
+        const previousCount = this.activityLogs.length;
+        
+        // Add timestamp to logs that don't have them
+        const processedLogs = serverLogs.map(log => ({
+          ...log,
+          timestamp: log.timestamp || new Date().toISOString()
+        }));
+        
+        this.activityLogs = processedLogs;
+        
+        console.log(`[SessionManager] ✅ Activity logs synced: ${previousCount} -> ${this.activityLogs.length} logs`);
+        
+        // 触发日志加载事件，让UI更新
+        this.emit('activityLogsLoaded', {
+          sessionId: this.currentSession.id,
+          logs: this.activityLogs
+        });
+      } else {
+        console.log(`[SessionManager] 📝 No activity logs found on server for session: ${this.currentSession.id}`);
+        this.activityLogs = [];
+      }
+    } catch (error) {
+      console.error(`[SessionManager] ❌ Failed to sync activity logs from server:`, error);
+      // 不抛出错误，允许任务提交继续进行
+    }
+  }
+
   async handleActivityUpdate(activityLog) {
     // Update current task status based on activity
     if (this.currentSession?.currentTask && activityLog.agent_status) {
@@ -562,11 +653,37 @@ class VibeSurfSessionManager {
 
   // Cleanup
   destroy() {
-    this.stopActivityPolling();
-    this.eventListeners.clear();
-    this.currentSession = null;
-    this.activityLogs = [];
+    // Prevent multiple cleanup calls
+    if (this.isDestroying) {
+      console.log('[SessionManager] Cleanup already in progress, skipping...');
+      return;
+    }
     
+    this.isDestroying = true;
+    console.log('[SessionManager] Destroying session manager...');
+    
+    try {
+      this.stopActivityPolling();
+      this.eventListeners.clear();
+      
+      // Clear any ongoing requests
+      if (this.pollingTimer) {
+        clearTimeout(this.pollingTimer);
+        this.pollingTimer = null;
+      }
+      
+      // Reset state
+      this.currentSession = null;
+      this.currentTaskId = null;
+      this.activityLogs = [];
+      this.isPolling = false;
+      
+      console.log('[SessionManager] Session manager cleanup complete');
+    } catch (error) {
+      console.error('[SessionManager] Error during destroy:', error);
+    } finally {
+      this.isDestroying = false;
+    }
   }
 
   // Status helpers
